@@ -15,7 +15,10 @@ from eventlet.support import get_errno
 DEFAULT_MAX_SIMULTANEOUS_REQUESTS = 1024
 DEFAULT_MAX_HTTP_VERSION = 'HTTP/1.1'
 MAX_REQUEST_LINE = 8192
+MAX_HEADER_LINE = 8192
+MAX_TOTAL_HEADER_SIZE = 65536
 MINIMUM_CHUNK_SIZE = 4096
+# %(client_port)s is also available
 DEFAULT_LOG_FORMAT= ('%(client_ip)s - - [%(date_time)s] "%(request_line)s"'
                      ' %(status_code)s %(body_length)s %(wall_seconds).6f')
 
@@ -163,6 +166,33 @@ class Input(object):
         return self.rfile._sock
 
 
+class HeaderLineTooLong(Exception):
+    pass
+
+
+class HeadersTooLarge(Exception):
+    pass
+
+
+class FileObjectForHeaders(object):
+
+    def __init__(self, fp):
+        self.fp = fp
+        self.total_header_size = 0
+
+    def readline(self, size=-1):
+        sz = size
+        if size < 0:
+            sz = MAX_HEADER_LINE
+        rv = self.fp.readline(sz)
+        if size < 0 and len(rv) >= MAX_HEADER_LINE:
+            raise HeaderLineTooLong()
+        self.total_header_size += len(rv)
+        if self.total_header_size > MAX_TOTAL_HEADER_SIZE:
+            raise HeadersTooLarge()
+        return rv
+
+
 class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
     protocol_version = 'HTTP/1.1'
     minimum_chunk_size = MINIMUM_CHUNK_SIZE
@@ -192,8 +222,8 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
             return
 
         try:
-            self.raw_requestline = self.rfile.readline(MAX_REQUEST_LINE)
-            if len(self.raw_requestline) == MAX_REQUEST_LINE:
+            self.raw_requestline = self.rfile.readline(self.server.url_length_limit)
+            if len(self.raw_requestline) == self.server.url_length_limit:
                 self.wfile.write(
                     "HTTP/1.0 414 Request URI Too Long\r\n"
                     "Connection: close\r\nContent-length: 0\r\n\r\n")
@@ -210,8 +240,25 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
             self.close_connection = 1
             return
 
-        if not self.parse_request():
+        orig_rfile = self.rfile
+        try:
+            self.rfile = FileObjectForHeaders(self.rfile)
+            if not self.parse_request():
+                return
+        except HeaderLineTooLong:
+            self.wfile.write(
+                "HTTP/1.0 400 Header Line Too Long\r\n"
+                "Connection: close\r\nContent-length: 0\r\n\r\n")
+            self.close_connection = 1
             return
+        except HeadersTooLarge:
+            self.wfile.write(
+                "HTTP/1.0 400 Headers Too Large\r\n"
+                "Connection: close\r\nContent-length: 0\r\n\r\n")
+            self.close_connection = 1
+            return
+        finally:
+            self.rfile = orig_rfile
 
         content_length = self.headers.getheader('content-length')
         if content_length:
@@ -384,13 +431,16 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
             for hook, args, kwargs in self.environ['eventlet.posthooks']:
                 hook(self.environ, *args, **kwargs)
 
-            self.server.log_message(self.server.log_format % dict(
-                client_ip=self.get_client_ip(),
-                date_time=self.log_date_time_string(),
-                request_line=self.requestline,
-                status_code=status_code[0],
-                body_length=length[0],
-                wall_seconds=finish - start))
+            if self.server.log_output:
+                self.server.log_message(self.server.log_format % {
+                    'client_ip': self.get_client_ip(),
+                    'client_port': self.client_address[1],
+                    'date_time': self.log_date_time_string(),
+                    'request_line': self.requestline,
+                    'status_code': status_code[0],
+                    'body_length': length[0],
+                    'wall_seconds': finish - start,
+                })
 
     def get_client_ip(self):
         client_ip = self.client_address[0]
@@ -406,6 +456,7 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
         env['SCRIPT_NAME'] = ''
 
         pq = self.path.split('?', 1)
+        env['RAW_PATH_INFO'] = pq[0]
         env['PATH_INFO'] = urllib.unquote(pq[0])
         if len(pq) > 1:
             env['QUERY_STRING'] = pq[1]
@@ -424,6 +475,7 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
         env['SERVER_NAME'] = host
         env['SERVER_PORT'] = str(port)
         env['REMOTE_ADDR'] = self.client_address[0]
+        env['REMOTE_PORT'] = str(self.client_address[1])
         env['GATEWAY_INTERFACE'] = 'CGI/1.1'
 
         for h in self.headers.headers:
@@ -453,7 +505,12 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
         return env
 
     def finish(self):
-        BaseHTTPServer.BaseHTTPRequestHandler.finish(self)
+        try:
+            BaseHTTPServer.BaseHTTPRequestHandler.finish(self)
+        except socket.error, e:
+            # Broken pipe, connection reset by peer
+            if get_errno(e) not in BROKEN_SOCK:
+                raise
         greenio.shutdown_safe(self.connection)
         self.connection.close()
 
@@ -471,7 +528,9 @@ class Server(BaseHTTPServer.HTTPServer):
                  minimum_chunk_size=None,
                  log_x_forwarded_for=True,
                  keepalive=True,
+                 log_output=True,
                  log_format=DEFAULT_LOG_FORMAT,
+                 url_length_limit=MAX_REQUEST_LINE,
                  debug=True):
 
         self.outstanding_requests = 0
@@ -490,7 +549,9 @@ class Server(BaseHTTPServer.HTTPServer):
         if minimum_chunk_size is not None:
             protocol.minimum_chunk_size = minimum_chunk_size
         self.log_x_forwarded_for = log_x_forwarded_for
+        self.log_output = log_output
         self.log_format = log_format
+        self.url_length_limit = url_length_limit
         self.debug = debug
 
     def get_environ(self):
@@ -537,7 +598,9 @@ def server(sock, site,
            log_x_forwarded_for=True,
            custom_pool=None,
            keepalive=True,
+           log_output=True,
            log_format=DEFAULT_LOG_FORMAT,
+           url_length_limit=MAX_REQUEST_LINE,
            debug=True):
     """  Start up a wsgi server handling requests from the supplied server
     socket.  This function loops forever.  The *sock* object will be closed after server exits,
@@ -556,7 +619,9 @@ def server(sock, site,
     :param log_x_forwarded_for: If True (the default), logs the contents of the x-forwarded-for header in addition to the actual client ip address in the 'client_ip' field of the log line.
     :param custom_pool: A custom GreenPool instance which is used to spawn client green threads.  If this is supplied, max_size is ignored.
     :param keepalive: If set to False, disables keepalives on the server; all connections will be closed after serving one request.
+    :param log_output: A Boolean indicating if the server will log data or not.
     :param log_format: A python format string that is used as the template to generate log lines.  The following values can be formatted into it: client_ip, date_time, request_line, status_code, body_length, wall_seconds.  The default is a good example of how to use it.
+    :param url_length_limit: A maximum allowed length of the request url. If exceeded, 414 error is returned.
     :param debug: True if the server should send exception tracebacks to the clients on 500 errors.  If False, the server will respond with empty bodies.
     """
     serv = Server(sock, sock.getsockname(),
@@ -567,7 +632,9 @@ def server(sock, site,
                   minimum_chunk_size=minimum_chunk_size,
                   log_x_forwarded_for=log_x_forwarded_for,
                   keepalive=keepalive,
+                  log_output=log_output,
                   log_format=log_format,
+                  url_length_limit=url_length_limit,
                   debug=debug)
     if server_event is not None:
         server_event.send(serv)
@@ -590,10 +657,13 @@ def server(sock, site,
                 port = ''
 
         serv.log.write("(%s) wsgi starting up on %s://%s%s/\n" % (
-            os.getpid(), scheme, host, port))
+            serv.pid, scheme, host, port))
         while True:
             try:
                 client_socket = sock.accept()
+                if debug:
+                    serv.log.write("(%s) accepted %r\n" % (
+                        serv.pid, client_socket[1]))
                 try:
                     pool.spawn_n(serv.process_request, client_socket)
                 except AttributeError:
